
library("limma")
library("Biobase")
library("tilingArray")

cit.load <-function (filename) 
{
    if (file.exists(filename)) 
        return(eval(parse(text = load(filename))))
    cat(paste("error - function cit.load : file ", filename, 
        " doesn't exist\n"))
    NULL
}

pdf2  <- function (file = NULL, width = 16.53, height = 11.69) 
{
    pdf(file = file, width = width, height = height)
}


factochar <- function (d) 
{
    for (i in 1:ncol(d)) if (is.factor(d[, i])) 
        d[, i] <- as.character(d[, i])
    d
}

 cit.sciFormat <-function (x, nbdec = 3, nmax = 4, pow = c("e", "10")) 
{
    pow = match.arg(pow)
    w <- which(!is.na(x))
    if (length(w) == 0) 
        return(x)
    L = strsplit(as.character(format(signif(x[w], nbdec), scientific = TRUE)), 
        split = "e")
    tmp <- NULL
    for (i in 1:length(L)) {
        if (abs(as.numeric(L[[i]][2])) < nmax) {
            tmp <- c(tmp, format(signif(x[w][i], nbdec), scientific = FALSE))
        }
        else {
            tmp <- c(tmp, format(signif(x[w][i], nbdec), scientific = TRUE))
        }
    }
    if (pow == "10") 
        tmp <- gsub("e", " 10", tmp)
    x[w] <- tmp
    x
}

cit.chromString2num   <-   function (d, chrom = "chrom", chromNum = "chrNum") 
{
    if (options("verbose")$verbose) 
        print("cit.chromString2num")
    if (chromNum %in% names(d)) {
        if (all(is.numeric(d[, chromNum]))) {
            if (options("verbose")$verbose) 
                warning("chromNum already available : will not be recalculated.")
            return(d)
        }
    }
    d[, chromNum] <- as.character(d[, chrom])
    maxchr <- suppressWarnings(max(as.numeric(d[, chromNum]), 
        na.rm = TRUE))
    w <- which(d[, chromNum] == "X")
    if (length(w) > 0) 
        d[w, chromNum] <- as.character(maxchr + 1)
    w <- which(d[, chromNum] == "Y")
    if (length(w) > 0) 
        d[w, chromNum] <- as.character(maxchr + 2)
    w <- which(!(d[, chromNum] %in% as.character(1:50)))
    if (length(w) > 0) 
        d[w, chromNum] <- NA
    d[, chromNum] <- as.numeric(d[, chromNum])
    d
}

genomorder <-  cit.genomOrder <-   function (d, chrom = "chrom", pos = "meanPos", id = NULL, startPos = "start", 
    endPos = "end", chromNum = "chrNum") 
{
    if (options("verbose")$verbose) 
        print("cit.genomOrder")
    d <- factochar(d)
    if (!chromNum %in% names(d)) 
        d <- cit.chromString2num(d, chrom = chrom, chromNum = chromNum)
    if (is.null(pos)) {
        pos <- "meanPos"
        if (options("verbose")$verbose) 
            warning("missing value for parameter pos -> using default value (meanPos).")
    }
    if (!pos %in% colnames(d)) {
        pos <- "meanPos"
        if (options("verbose")$verbose) 
            warning("uncorrect value for parameter pos -> using default value (meanPos).")
    }
    if (pos == "meanPos" & !"meanPos" %in% colnames(d)) {
        if (all(c(startPos, endPos) %in% colnames(d))) {
            d$meanPos <- rowMeans(d[, c(startPos, endPos)], na.rm = T)
        }
        else {
            stop("missing or uncorrect value for parameter pos or startPos + endPos.")
        }
    }
    ord <- order(d[, chromNum], as.numeric(d[, pos]), na.last = TRUE)
    d <- d[ord, ]
    if (!is.null(id)) {
        if (id %in% colnames(d)) 
            if (nrow(d) == length(unique(d[, id]))) 
                rownames(d) <- d[, id]
    }
    d
}


 cit.survCounts <- function (data = NULL, delay, strat = NULL, times)
{
    if (length(delay) == 1)
        delay = data[, delay]
    if (!is.null(strat) & length(strat) == 1)
        strat = data[, strat]
    survC <- NULL
    if (is.null(strat)) {
        for (tm in times) survC <- c(survC, length(which(delay >=
            tm)))
        names(survC) = as.character(times)
    }
    else {
        strat = as.factor(strat)
        names.strat = as.character(levels(strat))
        survC = matrix(NA, length(names.strat), length(times))
        for (iii in 1:length(names.strat)) {
            for (jjj in 1:length(times)) survC[iii, jjj] <- length(which(delay[which(as.character(strat) ==
                names.strat[iii])] >= times[jjj]))
        }
        survC = as.data.frame(survC)
        names(survC) = as.character(times)
        rownames(survC) = names.strat
    }
    return(survC)
}




  cit.cutDelay <-
function (data = NULL, event, delay, cutAfter = NULL, cutBefore = NULL,
    supervised = FALSE)
{
    name.delay = NULL
    if (length(delay) == 1) {
        name.delay = names(data)[delay]
        delay = data[, delay]
    }
    name.event = NULL
    if (length(event) == 1) {
        name.event = names(data)[event]
        event = data[, event]
    }
    if (!is.null(cutAfter)) {
        w = which(delay > cutAfter & !is.na(event))
        if (length(w) > 0) {
            delay[w] = cutAfter
            event[w] = 0
        }
        w = which(delay > cutAfter)
        if (length(w) > 0)
            delay[w] = cutAfter
    }
    if (!is.null(cutBefore)) {
        w = which(delay < cutBefore)
        if (length(w) > 0)
            event[w] = 0
    }
    res = as.data.frame(cbind(event, delay))
    if (!is.null(name.delay))
        names(res)[2] = name.delay
    if (!is.null(name.event))
        names(res)[1] = name.event
    if (supervised) {
        toNA = which(is.na(res[, 2]))
        res[toNA, 1] = NA
        toNA = which(res[, 2] < max(res[, 2], na.rm = TRUE) &
            res[, 1] == 0)
        res[toNA, 1] = NA
        res[toNA, 2] = NA
    }
    return(res)
}

cit.rainbow  <- function (n, default2Col = c("black", "red"), default6Col = c("red",
    "blue", "green", "orange", "purple", "gray"), colorMode = c("rainbow",
    "gray", "grey"))
{
    colorMode = match.arg(colorMode)
    if (colorMode == "grey")
        colorMode = "gray"
    if (n == 1)
        return(default2Col[1])
    if (n == 2)
        return(default2Col)
    if (n > 6 & colorMode == "rainbow")
        return(rainbow(n))
    if (n < 7 & n > 2 & colorMode == "rainbow")
        return(default6Col[1:n])
    if (n > 2 & colorMode == "gray")
        return(gray(seq(1, 0, length = n)))
}

 cit.surv    <- function (data = NULL, event, delay, var = NULL, strat = NULL,
    clust = NULL, survCounts = FALSE, times = NULL, na.action,
    method = c("efron", "breslow", "exact"), plot = TRUE, strat.colors = cit.rainbow,
    test = c("logrank", "likelihood", "wald"), timeby = 10, dec = 3,
    cutDelayAfter = NULL, cutDelayBefore = NULL, times.unit = NULL,
    posLegend = "bottomright",posTest="bottomleft", legend.cex = 0.75, lasAxis1 = 1,
    xlab = NULL, ylab = "frequency", ...)
{
    method = match.arg(method)
    test = match.arg(test)
    if (length(event) == 1) {
        name.event = names(data)[event]
        event = data[, event]
    }
    else {
        name.event = names(event)
    }
    if (length(delay) == 1)
        delay = data[, delay]
    if (length(var) == 1) {
        name.var = names(data)[var]
        var = data[, var]
    }
    else {
        name.var = names(var)
    }
    if (length(strat) == 1) {
        name.strat = names(data)[strat]
        strat = data[, strat]
    }
    else {
        name.strat = names(strat)
    }
    if (length(clust) == 1)
        clust = data[, clust]
    if (!is.null(cutDelayAfter) | !is.null(cutDelayBefore)) {
        cD = cit.cutDelay(event = event, delay = delay, cutAfter = cutDelayAfter,
            cutBefore = cutDelayBefore)
        event = cD[, 1]
        delay = cD[, 2]
    }
    survFormula = "Surv(delay, event)~"
    if (!is.null(var)) {
        survFormula = paste(survFormula, "var", sep = "")
        if (!is.null(strat))
            survFormula = paste(survFormula, "as.factor(strat)",
                sep = " + ")
    }
    else {
        if (!is.null(strat))
            survFormula = paste(survFormula, "as.factor(strat)",
                sep = "")
    }
    if (!is.null(clust))
        survFormula = paste(survFormula, "cluster(clust)", sep = " + ")
    coxMod = NULL
    if (!is.null(var) | !is.null(strat)) {
        survFormula = formula(survFormula)
        coxMod = coxph(survFormula, na.action = na.action, method = method)
    }
    sC = NULL
    if (survCounts) {
        if (is.null(times))
            times = seq(0, max(delay, na.rm = TRUE), by = 2 *
                timeby)
        sC = cit.survCounts(data = data, delay = delay, strat = strat,
            times = times)
        delta = 1
        DELTA = 0.2
        NB = length(levels(as.factor(strat)))
        par(mai = c(1.3 + (NB - 1) * DELTA, 0.75, 0.75, 0.75))
    }
    if (plot) {
        if (!is.null(strat)) {
            sf = survfit(Surv(delay, event) ~ strata(strat))
        }
        else sf = survfit(Surv(delay, event))
        nStrat = NULL
        if (is.null(strat))
            nStrat = 1
        else nStrat = length(levels(as.factor(strat)))
        if (is.null(xlab)) {
            xlab = "times"
            if (!is.null(times.unit))
                xlab = paste("times in", times.unit, sep = " ")
            if (survCounts)
                xlab = "times and counts"
        }
        if (!is.null(strat))
            plot(sf, col = strat.colors(nStrat), xlab = xlab,
                ylab = ylab, axes = FALSE, cex.main = 1, ...)
        else plot(sf, xlab = xlab, ylab = ylab, axes = FALSE,
            cex.main = 1, ...)
        axis(1, seq(0, max(delay, na.rm = TRUE), by = timeby),
            pos = NA, las = lasAxis1)
        AT <- c(0, 0.2, 0.4, 0.6, 0.8, 1)
        axis(2, AT, AT, las = 2)
        if (!is.null(strat)) {
            tmpStrat = strat[which(!is.na(event))]
            tStrat = table(tmpStrat)
            tStrat = tStrat[which(tStrat > 0)]
            nameStrat = attr(tStrat, "dimnames")[[1]]
            S <- split(rownames(data), as.character(strat))[levels(as.factor(strat))]
            annotLegend <- paste(nameStrat, " (n=", tStrat, ")",
                sep = "")
            legend(posLegend, inset = 0.01, cex = legend.cex,
                annotLegend, fill = strat.colors(nStrat)[match(nameStrat,
                  names(S))], bty = "o")
        }
        if (survCounts) {
            nameStrat = levels(as.factor(strat))
            for (xxx in 1:NB) {
                mtext(paste(nameStrat[xxx], " : ", sep = ""),
                  at = -timeby, side = 1, line = 4.95 + (xxx -
                    1) * delta, cex = 0.9, adj = 1)
                axis(at = times, labels = sC[xxx, ], side = 1,
                  line = 4 + (xxx - 1) * delta, cex.axis = 0.8,
                  tick = FALSE)
            }
        }
    }
    anytest = as.data.frame(matrix(NA, 3, 3))
    names(anytest) = c("likelihood", "logrank", "wald")
    rownames(anytest) = c("statistic", "df", "p.value")
    v.test = NULL
    s.test = NULL
    testLegend = NULL
    if (!is.null(var) | !is.null(strat))
        testLegend = c(paste(test, "test", sep = " "))
    if (!is.null(var)) {
        if (!is.null(strat)) {
            v.coxMod = coxph(Surv(delay, event) ~ var + strata(strat))
        }
        else {
            v.coxMod = coxMod
        }
        v.test = anytest
        v.test[, 1] = as.numeric(summary(v.coxMod)$logtest)
        v.test[, 2] = as.numeric(summary(v.coxMod)$sctest)
        v.test[, 3] = as.numeric(summary(v.coxMod)$waldtest)
        testLegend = c(testLegend, paste("var: p-value =", cit.sciFormat(v.test[3,
            test], dec), sep = " "))
    }
    if (!is.null(strat)) {
        s.coxMod = coxph(Surv(delay, event) ~ as.factor(strat))
        s.test = anytest
        s.test[, 1] = as.numeric(summary(s.coxMod)$logtest)
        s.test[, 2] = as.numeric(summary(s.coxMod)$sctest)
        s.test[, 3] = as.numeric(summary(s.coxMod)$waldtest)
        testLegend = c(testLegend, paste("p-value =", cit.sciFormat(s.test[3,
            test], dec), sep = " "))
    }
    if (plot & (!is.null(var) | !is.null(strat)))
        legend(posTest, inset = 0.01, cex = legend.cex,
            testLegend, bty = "n")
    if (is.null(coxMod))
        SF = sf
    else SF = survfit(coxMod)
    return(structure(list(coxph = coxMod, survfit = SF, counts = sC,
        var = v.test, strat = s.test)))
}





pdf2  <- function (file = NULL, width = 16.53, height = 11.69) 
{
    pdf(file = file, width = width, height = height)
}

cit.quantileNormalize  <- function (d, baseline = NULL){ 

    nr <- nrow(d)
    if (is.null(baseline)) 
        baseline <- quantile(as.vector(d), probs = seq(0.5, nr - 
            0.5, by = 1)/nr, na.rm = TRUE)
    for (i in 1:ncol(d)) {
        ok <- which(!is.na(d[, i]))
        nOk <- length(ok)
        if (nOk == nr) {
            d[order(d[, i]), i] <- baseline
        }
        else {
            if (nOk > 1) {
                d[ok[order(d[ok, i])], i] <- quantile(baseline, 
                  probs = seq(0.5, nOk - 0.5, by = 1)/nOk)
            }
            else {
                stop("Some columns have to few non NA values : these columns should be removed.")
            }
        }
    }
    return(d)
}

cit.chromString2num   <- function (d, chrom = "chrom", chromNum = "chrNum") 
{
    if (options("verbose")$verbose) 
        print("cit.chromString2num")
    if (chromNum %in% names(d)) {
        if (all(is.numeric(d[, chromNum]))) {
            if (options("verbose")$verbose) 
                warning("chromNum already available : will not be recalculated.")
            return(d)
        }
    }
    d[, chromNum] <- as.character(d[, chrom])
    maxchr <- suppressWarnings(max(as.numeric(d[, chromNum]), 
        na.rm = TRUE))
    w <- which(d[, chromNum] == "X")
    if (length(w) > 0) 
        d[w, chromNum] <- as.character(maxchr + 1)
    w <- which(d[, chromNum] == "Y")
    if (length(w) > 0) 
        d[w, chromNum] <- as.character(maxchr + 2)
    w <- which(!(d[, chromNum] %in% as.character(1:50)))
    if (length(w) > 0) 
        d[w, chromNum] <- NA
    d[, chromNum] <- as.numeric(d[, chromNum])
    d
}


cit.segmentState2cloneState <-function (segmentState, cloneInfo) 
{
    cloneState <- rep(NA, nrow(cloneInfo))
    numSegment <- rep(NA, nrow(cloneInfo))
    for (i in 1:nrow(segmentState)) {
        w <- which(cloneInfo$chrom == segmentState[i, "chrom"] & 
            cloneInfo$maploc >= segmentState[i, "loc.start"] & 
            cloneInfo$maploc <= segmentState[i, "loc.end"])
        cloneState[w] <- segmentState[i, "seg.mean"]
        numSegment[w] <- i
    }
    cloneState
}


cit.oneChipDNAcopy <- function (data, chrom = "chrom", pos = "meanPos") 
{
    if (options("verbose")$verbose) 
        print("cit.oneChipDNAcopy")
    require(DNAcopy)
    nna <- which(!is.na(data$norm) & !is.na(data[, chrom]) & 
        !is.na(data[, pos]))
    d <- data[nna, ]
    d <- cit.chromString2num(d, chrom = chrom, chromNum = "chrNum")
    CNA.object <- CNA(cbind(d$norm), d$chrNum, d[, pos], data.type = "logratio", 
        sampleid = "norm")
    smoothed.CNA.object <- smooth.CNA(CNA.object)
    segment.smoothed.CNA.object <- segment(smoothed.CNA.object)
    tmp <- segment.smoothed.CNA.object$output
    tmp <- split(tmp[, -1], tmp[, 1])
    data[nna, "smoot.DNAcopy"] <- cit.segmentState2cloneState(tmp[[1]], 
        segment.smoothed.CNA.object$data)
    return(data)
}

cit.oneChipTilingArray  <- function (data, maxSeg = 5, maxk = 200, criteria = "BIC", chrom = "chrom", 
    pos = "meanPos") 
{
    if (options("verbose")$verbose) 
        print("cit.oneChipTilingArray")
    chrom.uniq <- unique(data[, chrom])
    smoot <- rep(NA, nrow(data))
    counter = 0
    nna <- which(!is.na(data$norm) & !is.na(data[, chrom]) & 
        !is.na(data[, pos]))
    l2r <- data[nna, "norm"]
    ds <- rep(NA, length(nna))
    anna <- data[nna, ]
    for (j in chrom.uniq) {
        l2rC <- l2r[anna[, chrom] == j]
        if (length(l2rC) < 2) 
            next
        seg <- segment(l2rC, maxk = length(l2rC), maxseg = min(length(l2rC), 
            maxSeg))
        selected <- which.max(logLik(seg, penalty = criteria))
        est <- c(1, seg@breakpoints[[selected]][, 1], (length(l2rC) + 
            1))
        counter2 = 0
        statecounter = 1
        for (k in 2:length(est)) {
            ds[(counter + est[k - 1]):(counter + est[k] - 1)] <- mean(l2rC[(est[k - 
                1]):est[k] - 1])
            counter2 = est[k] - 1
            statecounter = statecounter + 1
        }
        counter = counter + (est[length(est)] - 1)
    }
    smoot[nna] <- ds
    data <- cbind(data, smoot.TilingArray = smoot)
    return(data)
}


decoupe <- function(z,Nmax=1000){
           if(length(z)<Nmax){
                       list(z )
                     }else{
                       k<- ceiling(length(z)/Nmax) 
                       
                       split(z,sort(rep(1:k,Nmax)[1:length(z)]))
                     }
}


cit.sdEpsi <-function (datanorm = NULL, datasmoot = NULL, minGroupSize = 100, ...) {
     

    if (options("verbose")$verbose) 
        print("cit.sdEpsi")
    if (is.null(datanorm) | is.null(datasmoot)) 
        stop("cit.sdEpsi - datanorm and datasmoot needed")
    if (is.vector(datanorm) & is.vector(datasmoot)) {
        if (length(datanorm) != length(datasmoot)) {
            stop("cit.sdEpsi - datanorm and datasmoot have different sizes")
        }
        Cc <- "V1"
    }
    else {
        Cc <- intersect(dimnames(datasmoot)[[2]], dimnames(datanorm)[[2]])
        Rc <- intersect(dimnames(datasmoot)[[1]], dimnames(datanorm)[[1]])
        if (length(Cc) < 2 | length(Rc) < minGroupSize) 
            stop("cit.sdEpsi - datanorm and datasmoot : not enough common rows or columns")
        datanorm <- datanorm[Rc, Cc]
        datasmoot <- datasmoot[Rc, Cc]
    }
    TableSdEpsi <- as.data.frame(array(dim = c(length(Cc), 9)))
    names(TableSdEpsi) <- c("sd.global", "sd.segm.biggestGroup", 
        "sd.segm.smallestSd", "sd.forced", "epsi.segm.medianBiggestGroup", 
        "epsi.segm.minBiggestGroup", "epsi.segm.maxBiggestGroup", 
        "epsi.global.mode", "epsi.forced")
    rownames(TableSdEpsi) <- Cc
    if (length(Cc) == 1) {
        TableSdEpsi[, ] <- rbind(cit.oneChipSdEpsi(datanorm = datanorm, 
            datasmoot = datasmoot, minGroupSize = minGroupSize, 
            ...)[[1]])
    }
    else {
        TableSdEpsi[, ] <- matrix(sapply(Cc, function(z) {
            cit.oneChipSdEpsi(datanorm = datanorm[, z], datasmoot = datasmoot[, 
                z], minGroupSize = minGroupSize, ...)[[1]]
        }), ncol = 9, byrow = TRUE)
    }
    TableSdEpsi
}

cit.discretize <- function (x, lim, quant = FALSE, addlevels = FALSE) 
{
    lim <- sort(lim)
    if (quant & (any(lim > 1) | any(lim < 0))) 
        stop("lim must be [0;1] as quant=T\n")
    res <- rep(NA, length(x))
    if (quant) 
        lim <- quantile(x, probs = lim, na.rm = TRUE)
    n <- length(lim)
    for (i in n:1) res[which(x < lim[i])] <- i
    res[which(x >= lim[n])] <- n + 1
    if (addlevels) {
        res <- as.factor(res)
        if (quant) 
            lim <- gsub(" ", "", prettyNum(lim, format = "g", 
                digits = 1))
        if (length(lim) == 1) 
            lev <- c(paste("<", lim[1], sep = ""), paste(">=", 
                lim[1], sep = ""))
        else lev <- c(paste("<", lim[1], sep = ""), paste("[", 
            lim[-length(lim)], ";", lim[-1], "[", sep = ""), 
            paste(">=", lim[length(lim)], sep = ""))
        levels(res) <- lev[as.numeric(levels(res))]
    }
    res
}

cit.density <- function (x, doplot = FALSE, pc = 0.05, ...) 
{
    dx <- density(x, na.rm = TRUE, ...)
    ymax <- diff(range(dx$y))
    n <- length(dx$y)
    croissance <- as.numeric((dx$y[-1] - dx$y[-n]) > 0)
    wB <- 1 + which(diff(croissance) == 1)
    wH <- 1 + which(diff(croissance) == -1)
    pointsBas <- dx$x[wB]
    pointsHauts <- dx$x[intersect(wH, which(dx$y > pc * ymax))]
    if (length(pointsHauts) > 0) 
        pointsBas <- sapply(split(pointsBas, cit.discretize(pointsBas, 
            pointsHauts)), median)
    if (doplot) {
        plot(dx, ...)
        abline(v = pointsBas, lty = 3, col = "blue")
        abline(v = pointsHauts, lty = 3, col = "red")
    }
    L <- c(dx, list(down = pointsBas, top = pointsHauts))
    attr(L, "class") <- "density"
    L
}


cit.oneChipSdEpsi <- function (datanorm = NULL, datasmoot = NULL, minGroupSize = 100, 
    sd. = NULL, epsi = NULL) 
{
    if (options("verbose")$verbose) 
        print("cit.oneChipSdEpsi")
    dsn <- datanorm - datasmoot
    categ <- rep(1, length(datasmoot))
    try(categ <- cit.discretize(datasmoot, cit.density(datasmoot)$down))
    L <- split(dsn, categ)
    Lsize <- sapply(L, length)
    x0 <- sd.na(dsn)
    w1 <- which.max(Lsize)
    x1 <- sd.na(L[[w1]])
    w2 <- which(Lsize >= minGroupSize)
    if (length(w2) == 0) {
        x2 <- NA
    }
    else {
        x2 <- min(sapply(L[w2], sd.na))
    }
    tmp <- split(datasmoot, categ)[[w1]]
    x3 <- median(tmp)
    x4 <- min(tmp)
    x5 <- max(tmp)
    x6 <- cit.mode(datanorm)
    if (is.null(sd.)) 
        sd. <- NA
    if (is.null(epsi)) 
        epsi <- NA
    list(c(sd.global = x0, sd.segm.biggestGroup = x1, sd.segm.smallestSd = x2, 
        sd.forced = sd., epsi.segm.medianBiggestGroup = x3, epsi.segm.minBiggestGroup = x4, 
        epsi.segm.maxBiggestGroup = x5, epsi.global.mode = x6, 
        epsi.forced = epsi), categ = categ)
}


cit.mode <-  function (x) 
{
    x <- x[!is.na(x)]
    d <- density(x)
    i <- which.max(d$y)
    d$x[i]
}
 
    


cit.smooth <- function(valueNumchromPos,funSmooth=cit.oneChipTilingArray){
    valueNumchromPos <- valueNumchromPos[order(valueNumchromPos[,2],valueNumchromPos[,3]),]
    
    L <- split(1:nrow(valueNumchromPos),valueNumchromPos[,2])
    
    
    valueNumchromPos[unlist(L),"smoot"] <- unlist(lapply(L,function(z) {
                      
                   funSmooth( valueNumchromPos[z,])[,4]
                   }))
    valueNumchromPos
}

cit.getGNL  <- function (datanorm = NULL, datasmoot = NULL, sd. = NULL, epsi = NULL, 
    epsiRange = NULL, NsdForGL = 1, NsdForA = 3) 
{
    if (options("verbose")$verbose) 
        print("cit.getGNL")
    if (is.null(epsi) | is.null(sd.) | is.null(epsiRange)) {
        tmp <- cit.sdEpsi(datanorm = datanorm, datasmoot = datasmoot)
        if (is.null(epsi)) 
            epsi <- tmp[1, "epsi.segm.medianBiggestGroup"]
        if (is.null(sd.)) 
            sd. <- tmp[1, "sd.segm.biggestGroup"]
        if (is.null(epsiRange)) 
            epsiRange <- c(tmp[1, "epsi.segm.minBiggestGroup"], 
                tmp[1, "epsi.segm.maxBiggestGroup"])
    }
    res <- as.data.frame(array(dim = c(length(datasmoot), 2)))
    names(res) <- c("GNL=default", "GNL=stringent")
    w <- which(abs(datasmoot - epsi) < sd. * NsdForGL)
    if (length(w) > 0) 
        res[w, 1] <- rep(0, length(w))
    w <- which(datasmoot - epsi <= -sd. * NsdForGL)
    if (length(w) > 0) 
        res[w, 1] <- rep(-1, length(w))
    w <- which(datasmoot - epsi >= sd. * NsdForGL)
    if (length(w) > 0) 
        res[w, 1] <- rep(1, length(w))
    w <- which(datanorm - epsi <= -sd. * NsdForA)
    if (length(w) > 0) 
        res[w, 1] <- rep(-1, length(w))
    w <- which(datanorm - epsi >= sd. * NsdForA)
    if (length(w) > 0) 
        res[w, 1] <- rep(1, length(w))
    if (is.null(epsiRange)) 
        epsiRange <- c(epsi, epsi)
    epsiRange <- sort(epsiRange)
    w <- which(datasmoot - epsiRange[1] > -sd. * NsdForGL & datasmoot - 
        epsiRange[2] < sd. * NsdForGL)
    if (length(w) > 0) 
        res[w, 2] <- rep(0, length(w))
    w <- which(datasmoot - epsiRange[1] <= -sd. * NsdForGL)
    if (length(w) > 0) 
        res[w, 2] <- rep(-1, length(w))
    w <- which(datasmoot - epsiRange[2] >= sd. * NsdForGL)
    if (length(w) > 0) 
        res[w, 2] <- rep(1, length(w))
    w <- which(datanorm - epsiRange[1] <= -sd. * NsdForA)
    if (length(w) > 0) 
        res[w, 2] <- rep(-1, length(w))
    w <- which(datanorm - epsiRange[2] >= sd. * NsdForA)
    if (length(w) > 0) 
        res[w, 2] <- rep(1, length(w))
    w <- which((res[, 1] == 1 & res[, 2] == -1) | (res[, 1] == 
        -1 & res[, 2] == 1))
    if (length(w) > 0) 
        res[w, 2] <- rep(0, length(w))
    attributes(res)$"GNL parameters" = list(epsi = epsi, sd = sd., 
        NsdForGL = NsdForGL, NsdForA = NsdForA, epsiRange = epsiRange)
    res
}



cit.pangenomCoord <-function (x, chrom = "chrom", pos = "meanPos", startPos = "start", 
    endPos = "end", cyto = NULL, absPos = "absPos", 
    chromNum = "chrNum", id = NULL) 
{
    if (options("verbose")$verbose) 
        print("cit.pangenomCoord")
    x <- cit.genomOrder(d = x, chrom = chrom, pos = pos, id = id, 
        startPos = startPos, endPos = endPos, chromNum = chromNum)
    if (absPos %in% colnames(x)) {
        return(x)
    }
    if (is.null(endPos)) {
        endPos <- "endPos"
        if (options("verbose")$verbose) 
            warning("missing value for parameter endPos -> using default value (endPos).")
    }
    if (!endPos %in% colnames(x)) {
        endPos <- "endPos"
        if (pos %in% colnames(x)) {
            x[, endPos] <- x[, pos]
            if (options("verbose")$verbose) 
                warning("uncorrect value for parameter endPos -> using pos value.")
        }
        else {
            stop("missing value for pos or endPos.")
        }
    }
    if (is.null(absPos)) {
        absPos <- "absPos"
        if (options("verbose")$verbose) 
            warning("missing value for parameter absPos -> using default value.")
    }
    if (!absPos %in% names(x)) {
        x$absPos <- as.numeric(x[, pos])
        if (pos %in% colnames(x)) {
            x[, absPos] <- as.numeric(x[, pos])
            if (options("verbose")$verbose) 
                warning("uncorrect value for parameter absPos -> using pos value.")
        }
        else {
            stop("missing value for absPos or pos.")
        }
    }
    if (!is.null(cyto)) {
        if (!"ChrNumeric" %in% names(cyto)) 
            cyto <- cit.chromString2num(cyto, chrom = "Chromosome", 
                chromNum = "ChrNumeric")
        allchrnum <- sort(unique(c(cyto$ChrNumeric, x[, chromNum])))
        maxByChr <- sapply(allchrnum, function(z) {
            m <- 0
            w <- which(cyto$ChrNumeric == z)
            if (length(w) > 0) 
                m <- max(cyto[w, "End"], na.rm = TRUE)
            w <- which(x[, chromNum] == z)
            if (length(w) > 0) 
                m <- max(c(m, max(x[w, endPos], na.rm = TRUE)))
            m
        })
        maxByChr <- cumsum(as.numeric(maxByChr))
    }
    else {
        maxByChr <- cumsum(as.numeric(sapply(split(x[, endPos], 
            x[, chromNum]), max)))
    }
    for (i in setdiff(unique(x[, chromNum]), 1)) {
        w <- which(x[, chromNum] == i)
        x[w, absPos] <- x[w, absPos] + maxByChr[i - 1]
    }
    w <- which(is.na(x[, chromNum]))
    if (length(w) > 0) 
        x[w, absPos] <- NA
    x
}

 cit.gnl2color  <- function (x, colorsLNG = c("cyan", "green", "yellow", "red", 
    "magenta")) 
{
    wL2 <- which(x == -2)
    wL <- which(x == -1)
    wN <- which(x == 0)
    wG <- which(x == 1)
    wG2 <- which(x == 2)
    if (length(wL2) > 0) 
        x[wL2] <- colorsLNG[1]
    if (length(wL) > 0) 
        x[wL] <- colorsLNG[2]
    if (length(wN) > 0) 
        x[wN] <- colorsLNG[3]
    if (length(wG) > 0) 
        x[wG] <- colorsLNG[4]
    if (length(wG2) > 0) 
        x[wG2] <- colorsLNG[5]
    x
}

cit.pangenomPlotMore <-
function (d = NULL, ycol = NULL, ytransform = NULL, reverseYaxis = FALSE, 
    colorscol = NULL, colortransform = NULL, chrom = "chrom", 
    pos = "meanPos", startPos = "start", endPos = "end", rectangleOption = FALSE, 
    cyto = cit.data("cytoband"), absPos = "absPos", textInfoCol = NULL, 
    textInfoFilterCol = NULL, textInfoColTransform = NULL, plotCentro = TRUE, 
    plotChrom = TRUE, plotCyto = TRUE, plotnew = TRUE, plotyaxis = TRUE, 
    yaxmark = NULL, yaxlab = NULL, col0 = 1, pages = c("all", 
        1:22, "X", "Y"), ...) 
{
    if (options("verbose")$verbose) 
        print("cit.pangenomPlotMore")
    if (!ycol %in% names(d)) 
        stop("Column ycol ", ycol, " not found in d.\n")
    d <- cit.pangenomCoord(d, chrom = chrom, pos = pos, startPos = startPos, 
        endPos = endPos, cyto = cyto, absPos = absPos)
    if ("all" %in% pages) {
        tmp <- cit.pangenomPlot(d = d, ycol = ycol, ytransform = ytransform, 
            reverseYaxis = reverseYaxis, colorscol = colorscol, 
            colortransform = colortransform, chrom = chrom, pos = pos, 
            startPos = startPos, endPos = endPos, rectangleOption = rectangleOption, 
            cyto = cyto, absPos = absPos, plotCentro = plotCentro, 
            plotChrom = plotChrom, plotnew = plotnew, plotyaxis = plotyaxis, 
            yaxmark = yaxmark, yaxlab = yaxlab, col0 = col0, 
            ...)
    }
    pages <- setdiff(pages, "all")
    for (pa in pages) {
        tmp <- cit.chromPlot(d = d, ycol = ycol, ytransform = ytransform, 
            reverseYaxis = reverseYaxis, colorscol = colorscol, 
            colortransform = colortransform, chromToPlot = pa, 
            chrom = chrom, pos = pos, startPos = startPos, endPos = endPos, 
            rectangleOption = rectangleOption, cyto = cyto, textInfoCol = textInfoCol, 
            textInfoFilterCol = textInfoFilterCol, textInfoColTransform = textInfoColTransform, 
            plotCentro = plotCentro, plotCyto = plotCyto, plotnew = plotnew, 
            plotyaxis = plotyaxis, yaxmark = yaxmark, yaxlab = yaxlab, 
            col0 = col0, ...)
        mtext(paste("chr", pa), line = 0.5)
    }
    d
}

cit.pangenomPlot <-   function (d = NULL, ycol = NULL, ytransform = NULL, reverseYaxis = FALSE, 
    colorscol = NULL, colortransform = NULL, col0 = 1, chrom = "chrom", 
    pos = "meanPos", startPos = "start", endPos = "end", cyto = cit.data("cytoband"), 
    absPos = "absPos", rectangleOption = FALSE, plotCentro = TRUE, 
    plotCentro.color = "blue", plotChrom = TRUE, plotnew = TRUE, 
    plotyaxis = TRUE, yaxmark = NULL, yaxlab = NULL, chromToPlot = c(1:22, 
        "X", "Y"), xlim = NULL, ...) 
{
    if (options("verbose")$verbose) 
        print("cit.pangenomPlot")
    if (!ycol %in% names(d)) 
        stop("ycol ", ycol, " not found in d.\n")
    d <- cit.pangenomCoord(d, chrom = chrom, pos = pos, startPos = startPos, 
        endPos = endPos)
    y <- d[, ycol]
    if (!is.null(ytransform)) 
        y <- ytransform(y)
    if (is.null(colortransform)) 
        colortransform <- function(z) {
            z
        }
    if (is.null(col0)) 
        col0 <- 1
    if (is.null(colorscol)) {
        colorY <- col0
    }
    else {
        if (!colorscol %in% names(d)) 
            colorY <- colorscol
        else colorY <- colortransform(d[, colorscol])
    }
    if (!is.null(cyto)) {
        if (!is.null(chromToPlot)) 
            cyto <- cyto[which(cyto$Chromosome %in% chromToPlot), 
                ]
        cyto <- cit.pangenomCoord(cyto, chrom = "ChrNumeric", 
            pos = "End", startPos = "Start", endPos = "End", 
            cyto = NULL, absPos = "absPos")
        if (is.null(xlim)) {
            xlim <- c(0, max(cyto$absPos, na.rm = T))
            xlim <- cit.majorXlim(xlim)
        }
    }
    else {
        if (is.null(xlim)) {
            xlim <- c(0, 3076781887)
            xlim <- cit.majorXlim(xlim)
        }
    }
    if (plotnew) {
        if (rectangleOption) {
            d$deltaPos <- abs(d[, endPos] - d[, startPos])/2
            plot(d[, absPos], -y, col = "white", axes = FALSE, 
                xlim = xlim, ...)
            abline(h = 0)
            if (reverseYaxis) {
                rect(d[, absPos] - d$deltaPos, -y, d[, absPos] + 
                  d$deltaPos, rep(0, nrow(d)), col = colorY, 
                  ...)
            }
            else {
                rect(d[, absPos] - d$deltaPos, y, d[, absPos] + 
                  d$deltaPos, rep(0, nrow(d)), col = colorY, 
                  ...)
            }
        }
        else {
            if (reverseYaxis) {
                plot(d[, absPos], -y, axes = FALSE, col = colorY, 
                  xlim = xlim, ...)
            }
            else {
                plot(d[, absPos], y, axes = FALSE, col = colorY, 
                  xlim = xlim, ...)
            }
        }
        box()
        if (plotyaxis) {
            okk <- FALSE
            if (is.null(yaxmark)) {
                yaxmark <- pretty(y, n = 10)
                okk <- TRUE
            }
            if (is.null(yaxlab)) 
                yaxlab <- yaxmark
            if (reverseYaxis & okk) 
                yaxmark <- -yaxmark
            axis(2, at = yaxmark, labels = yaxlab, cex.axis = 0.8, 
                las = 2)
        }
    }
    else {
        if (rectangleOption) {
            d$deltaPos <- abs(d[, endPos] - d[, startPos])/2
            if (reverseYaxis) {
                rect(d[, absPos] - d$deltaPos, -y, d[, absPos] + 
                  d$deltaPos, rep(0, nrow(d)), col = colorY, 
                  ...)
            }
            else {
                rect(d[, absPos] - d$deltaPos, y, d[, absPos] + 
                  d$deltaPos, rep(0, nrow(d)), col = colorY, 
                  ...)
            }
        }
        else {
            if (reverseYaxis) {
                points(d[, absPos], -y, col = colorY, ...)
            }
            else {
                points(d[, absPos], y, col = colorY, ...)
            }
        }
    }
    wat <- NULL
    if (!is.null(cyto)) {
        w <- which(cyto$Centro == 1)
        wat <- sapply(split(cyto[w, "absPos"], cyto[w, "ChrNumeric"]), 
            mean)
        if (length(w) == 0) 
            wat <- sapply(split(cyto[, "absPos"], cyto[, "ChrNumeric"]), 
                mean)
        if (plotCentro & length(w) != 0) 
            abline(v = wat, lty = 3, col = plotCentro.color)
        if (plotChrom) {
            axis(1, at = wat, labels = c(1:22, "X", "Y")[c(1:22, 
                "X", "Y") %in% unique(cyto$Chromosome)], cex.axis = 0.8, 
                las = 2)
            wat <- sapply(split(cyto[, "absPos"], cyto$ChrNumeric), 
                max)
            abline(v = wat[-length(wat)], lty = 1)
            if (length(w) == 0) 
                abline(h = 0, lty = 1)
        }
    }
    d
}


cit.majorXlim <- function (Xlim, pcent = 0.02) 
{
    delta <- diff(Xlim) * pcent
    cit.exactXlim(Xlim + c(-delta, delta))
}


cit.chromPlot   <- function (d = NULL, ycol = NULL, ytransform = NULL, reverseYaxis = FALSE, 
    colorscol = NULL, colortransform = NULL, chromToPlot = c(1:22, 
        "X", "Y")[1], chrom = "chrom", pos = "meanPos", startPos = "start", 
    endPos = "end", rectangleOption = FALSE, segmentOption = FALSE, 
    cyto = cit.data("cytoband"), textInfoCol = NULL, textInfoFilterCol = NULL, 
    textInfoColTransform = NULL, plotCentro = TRUE, plotCentro.color = "blue", 
    plotCyto = TRUE, plotnew = TRUE, plotyaxis = TRUE, yaxmark = NULL, 
    yaxlab = NULL, col0 = 1, xlim = NULL, ylim = NULL, ...) 
{
    if (options("verbose")$verbose) 
        print("cit.chromPlot")
    wc <- which(d[, chrom] == chromToPlot)
    if (length(wc) == 0) 
        return(NULL)
    d <- d[wc, ]
    if (is.null(pos)) {
        d$meanPos <- rowMeans(d[, c(startPos, endPos)])
        pos <- "meanPos"
    }
    d <- d[order(d[, pos]), ]
    y <- d[, ycol]
    if (!is.null(ytransform)) 
        y <- ytransform(y)
    if (is.null(colortransform)) 
        colortransform <- function(z) {
            z
        }
    if (is.null(col0)) 
        col0 <- 1
    if (is.null(colorscol)) {
        colorY <- col0
    }
    else {
        if (!colorscol %in% names(d)) 
            colorY <- col0
        else colorY <- colortransform(d[, colorscol])
    }
    if (!is.null(cyto)) {
        cyto <- cit.pangenomCoord(cyto, chrom = "Chromosome", 
            pos = "End", startPos = "Start", endPos = "End", 
            cyto = NULL, absPos = "absPos")
    }
    if (plotnew) {
        if (!is.null(cyto)) {
            if (is.null(xlim)) {
                xlim <- range(unlist(cyto[which(cyto$Chromosome == 
                  chromToPlot), c("Start", "End")]))
                xlim <- cit.majorXlim(xlim)
            }
        }
        else {
            if (is.null(xlim)) {
                xlim <- c(0, max(d[, endPos]))
                xlim <- cit.majorXlim(xlim)
            }
        }
        if (is.null(ylim)) 
            ylim <- range(ifelse(reverseYaxis, -1, 1) * y)
        plot(xlim, ylim, axes = FALSE, col = "white", xlim = xlim, 
            ylim = ylim, ...)
        if (rectangleOption) {
            abline(h = 0)
            if (reverseYaxis) {
                rect(d[, startPos], -y, d[, endPos], rep(0, nrow(d)), 
                  col = colorY, ...)
            }
            else {
                rect(d[, startPos], y, d[, endPos], rep(0, nrow(d)), 
                  col = colorY, ...)
            }
        }
        else {
            if (segmentOption) {
                if (reverseYaxis) {
                  rect(d[, startPos], -y, d[, endPos], -y, border = colorY, 
                    ...)
                }
                else {
                  rect(d[, startPos], y, d[, endPos], y, border = colorY, 
                    ...)
                }
            }
            else {
                if (reverseYaxis) {
                  points(d[, pos], -y, col = colorY, ...)
                }
                else {
                  points(d[, pos], y, col = colorY, ...)
                }
            }
        }
        box()
        if (plotyaxis) {
            okk <- FALSE
            if (is.null(yaxmark)) {
                yaxmark <- pretty(y, n = 10)
                okk <- TRUE
            }
            if (is.null(yaxlab)) 
                yaxlab <- yaxmark
            if (reverseYaxis & okk) 
                yaxmark <- -yaxmark
            axis(2, at = yaxmark, labels = yaxlab, las = 2)
        }
    }
    else {
        if (rectangleOption) {
            if (reverseYaxis) {
                rect(d[, startPos], -y, d[, endPos], rep(0, nrow(d)), 
                  col = colorY, ...)
            }
            else {
                rect(d[, startPos], y, d[, endPos], rep(0, nrow(d)), 
                  col = colorY, ...)
            }
        }
        else {
            if (segmentOption) {
                if (reverseYaxis) {
                  rect(d[, startPos], -y, d[, endPos], -y, border = colorY, 
                    ...)
                }
                else {
                  rect(d[, startPos], y, d[, endPos], y, border = colorY, 
                    ...)
                }
            }
            else {
                if (reverseYaxis) {
                  points(d[, pos], -y, col = colorY, ...)
                }
                else {
                  points(d[, pos], y, col = colorY, ...)
                }
            }
        }
    }
    if (!is.null(cyto)) {
        if (plotCentro) {
            w <- which(cyto$Centro == 1 & cyto$Chromosome == 
                chromToPlot)
            wat <- mean(as.numeric(apply(cyto[w, c("Start", "End")], 
                1, mean)))
            abline(v = wat, lty = 1, col = plotCentro.color)
        }
        if (plotCyto) {
            w <- which(cyto$Chromosome == chromToPlot)
            wat <- as.numeric(apply(cyto[w, c("Start", "End")], 
                1, mean))
            axis(1, at = wat, labels = cyto[w, "Band"], las = 3)
            if (plotCentro) 
                w <- which(cyto$Chromosome == chromToPlot & cyto$Centro != 
                  1)
            abline(v = cyto[w, "End"], lty = 3, col = "black")
        }
    }
    if (!is.null(textInfoCol)) 
        if (textInfoCol %in% names(d)) {
            w <- 1:nrow(d)
            if (!is.null(textInfoFilterCol)) 
                if (textInfoFilterCol %in% names(d)) 
                  w <- which(as.logical(d[, textInfoFilterCol]))
            if (length(w) > 0) {
                labs <- as.character(d[w, textInfoCol])
                if (!is.null(textInfoColTransform)) 
                  labs <- textInfoColTransform(labs)
                if (reverseYaxis) {
                  text(d[w, pos], -y[w], labs, ...)
                }
                else {
                  text(d[w, pos], y[w], labs, ...)
                }
            }
        }
}
